// #include "NvInfer.h"
#include "NvInferPlugin.h"
#include "cuda_runtime_api.h"
#include "decode.h"
// #include "logging.h"

// stuff we know about the network and the input/output blobs
// static const int INPUT_H = decodeplugin::INPUT_H;  // H, W must be able to  be divided by 32.
// static const int INPUT_W = decodeplugin::INPUT_W;;
// static const int OUTPUT_SIZE = (INPUT_H / 8 * INPUT_W / 8 + INPUT_H / 16 * INPUT_W / 16 + INPUT_H / 32 * INPUT_W / 32) * 2  * 15 + 1;
// const char* INPUT_BLOB_NAME = "data";
// const char* OUTPUT_BLOB_NAME = "prob";

using namespace nvinfer1;
// static Logger gLogger;

DecodePlugin::DecodePlugin()
{
}

DecodePlugin::~DecodePlugin()
{
}

// create the plugin at runtime from a byte stream
DecodePlugin::DecodePlugin(const void* data, size_t length)
{
}

void DecodePlugin::serialize(void* buffer) const
{
    // printf("use serialize\n");
}

size_t DecodePlugin::getSerializationSize() const
{  
    // printf("use getSerializationSize\n");
    return 0;
}

int DecodePlugin::initialize()
{ 
    return 0;
}

Dims DecodePlugin::getOutputDimensions(int index, const Dims* inputs, int nbInputDims)
{
    //output the result to channel
    int totalCount = 0;
    totalCount += decodeplugin::INPUT_H / 8 * decodeplugin::INPUT_W / 8 * 2 * sizeof(decodeplugin::Detection) / sizeof(float);
    totalCount += decodeplugin::INPUT_H / 16 * decodeplugin::INPUT_W / 16 * 2 * sizeof(decodeplugin::Detection) / sizeof(float);
    totalCount += decodeplugin::INPUT_H / 32 * decodeplugin::INPUT_W / 32 * 2 * sizeof(decodeplugin::Detection) / sizeof(float);

    // printf("totalCount %d", totalCount);

    return Dims3(totalCount + 1, 1, 1);
}

// Set plugin namespace
void DecodePlugin::setPluginNamespace(const char* pluginNamespace)
{
    mPluginNamespace = pluginNamespace;
}

const char* DecodePlugin::getPluginNamespace() const
{
    return mPluginNamespace;
}

// Return the DataType of the plugin output at the requested index
DataType DecodePlugin::getOutputDataType(int index, const nvinfer1::DataType* inputTypes, int nbInputs) const
{
    return DataType::kFLOAT;
}

// Return true if output tensor is broadcast across a batch.
bool DecodePlugin::isOutputBroadcastAcrossBatch(int outputIndex, const bool* inputIsBroadcasted, int nbInputs) const
{
    return false;
}

// Return true if plugin can use input that is broadcast across batch without replication.
bool DecodePlugin::canBroadcastInputAcrossBatch(int inputIndex) const
{
    return false;
}

void DecodePlugin::configurePlugin(const PluginTensorDesc* in, int nbInput, const PluginTensorDesc* out, int nbOutput)
{
}

// Attach the plugin object to an execution context and grant the plugin the access to some context resource.
void DecodePlugin::attachToContext(cudnnContext* cudnnContext, cublasContext* cublasContext, IGpuAllocator* gpuAllocator)
{
}

// Detach the plugin object from its execution context.
void DecodePlugin::detachFromContext() {}

const char* DecodePlugin::getPluginType() const
{
    return "Decode_TRT";
}

const char* DecodePlugin::getPluginVersion() const
{
    return "1";
}

void DecodePlugin::destroy()
{
    delete this;
}

// Clone the plugin
IPluginV2IOExt* DecodePlugin::clone() const
{
    // printf("use clone\n");
    DecodePlugin *p = new DecodePlugin();
    p->setPluginNamespace(mPluginNamespace);
    return p;
}

int DecodePlugin::enqueue(int batchSize, const void*const * inputs, void** outputs, void* workspace, cudaStream_t stream)
{
    //assert(batchSize == 1);
    //GPU
    //CUDA_CHECK(cudaStreamSynchronize(stream));
    // printf("Use enqueue.\n");
    forwardGpu((const float *const *)inputs, (float *)outputs[0], stream, batchSize);

    return 0;
};

DecodePluginCreator::DecodePluginCreator()
{
    mPluginAttributes.clear();

    mFC.nbFields = mPluginAttributes.size();
    mFC.fields = mPluginAttributes.data();
}

const char* DecodePluginCreator::getPluginName() const
{
    return "Decode_TRT";
}

const char* DecodePluginCreator::getPluginVersion() const
{
    return "1";
}

const PluginFieldCollection* DecodePluginCreator::getFieldNames()
{
    return &mFC;
}

IPluginV2IOExt* DecodePluginCreator::createPlugin(const char* name, const PluginFieldCollection* fc)
{
    DecodePlugin* obj = new DecodePlugin();
    obj->setPluginNamespace(mNamespace.c_str());
    return obj;
}

IPluginV2IOExt* DecodePluginCreator::deserializePlugin(const char* name, const void* serialData, size_t serialLength)
{
    // This object will be deleted when the network is destroyed, which will
    // call PReluPlugin::destroy()
    DecodePlugin* obj = new DecodePlugin(serialData, serialLength);
    obj->setPluginNamespace(mNamespace.c_str());
    return obj;
}

PluginFieldCollection DecodePluginCreator::mFC{};
std::vector<PluginField> DecodePluginCreator::mPluginAttributes;

REGISTER_TENSORRT_PLUGIN(DecodePluginCreator);
