/*
 * Copyright (c) 2019, NVIDIA CORPORATION. All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include <algorithm>
#include <cassert>
#include <cmath>
#include <cstring>
#include <fstream>
#include <iostream>
#include <unordered_map>
#include "nvdsinfer_custom_impl.h"

static bool first = true; 

extern "C" bool NvDsInferParseCustomRFace(
    std::vector<NvDsInferLayerInfo> const& outputLayersInfo,
    NvDsInferNetworkInfo const& networkInfo,
    NvDsInferParseDetectionParams const& detectionParams,
    std::vector<NvDsInferParseObjectInfo>& objectList);

extern "C" bool NvDsInferParseCustomRFace(
    std::vector<NvDsInferLayerInfo> const& outputLayersInfo,
    NvDsInferNetworkInfo const& networkInfo,
    NvDsInferParseDetectionParams const& detectionParams,
    std::vector<NvDsInferParseObjectInfo>& objectList)
{
    assert(outputLayersInfo.size() == 1);

    float *outputBuffer = (float *)outputLayersInfo[0].buffer;
    int bboxNum = static_cast<int>(outputBuffer[0]);
    for (int i = 0; i < bboxNum; ++i) {
        float confidence = outputBuffer[15 * i + 1 + 4];
        if (confidence < 0.1)
            continue;

        float rectX1Float, rectY1Float, rectX2Float, rectY2Float;
        rectX1Float = outputBuffer[15 * i + 1];
        rectY1Float = outputBuffer[15 * i + 2];
        rectX2Float = outputBuffer[15 * i + 3];
        rectY2Float = outputBuffer[15 * i + 4];

        if (rectX1Float >= (int)networkInfo.width)
            rectX1Float = networkInfo.width - 1;
        if (rectX2Float >= (int)networkInfo.width)
            rectX2Float = networkInfo.width - 1;
        if (rectY1Float >= (int)networkInfo.height)
            rectY1Float = networkInfo.height - 1;
        if (rectY2Float >= (int)networkInfo.height)
            rectY2Float = networkInfo.height - 1;

        if (rectX1Float < 0)
            rectX1Float = 0;
        if (rectX2Float < 0)
            rectX2Float = 0;
        if (rectY1Float < 0)
            rectY1Float = 0;
        if (rectY2Float < 0)
            rectY2Float = 0;

        if(((rectX2Float - rectX1Float) < 0) || ((rectY2Float - rectY1Float) < 0))
            continue;

        objectList.push_back({ 0, rectX1Float, rectY1Float, (rectX2Float - rectX1Float),
                    (rectY2Float - rectY1Float), confidence});
    }
    return true;
}

CHECK_CUSTOM_PARSE_FUNC_PROTOTYPE(NvDsInferParseCustomRFace);
